class OauthController < ResponseController

  before_action :set_state, only: :authorize
  before_action :check_state, only: [:oauth_verify, :openid_verify]

  def authorize
    if params[:name].present? && params[:type].present?
      set_oauth params[:name], params[:type]
    else
      ActiveRecord::Base.logger.error('имя и тип соц.сети не пришли')
      redirect_to authorization_url, flash: {pills: 'имя и тип соц.сети не пришли'}
    end

  # отправка пользователя на подтверждение
    redirect_to @oauth.auth_request
  end

  def oauth_verify
    if params[:code].present?
      # обмен code на токен
      access_card_params = @oauth.auth_request :step2, {:code => params[:code]}

      case access_card_params[:state]
        when :ok
          sign_in access_card_params
          redirect_to profile_path
        else
          # данные не обработаны по причине access_card_params[:error]
          ActiveRecord::Base.logger.error("данные не обработаны по причине #{access_card_params[:error]}")
          # ActionCable.server.broadcast "pills", {state: 'red', message: "данные не обработаны по причине #{access_card_params[:error]}", url: authorization_url}
          redirect_to authorization_url, flash: {pills: "данные не обработаны по причине #{access_card_params[:error]}"}
      end
    else
      #code не пришёл от api соц.сети
      ActiveRecord::Base.logger.error("code не пришёл от api соц.сети")
      redirect_to authorization_url, flash: {pills: 'code не пришёл от api соц.сети'}
    end
  end

  def openid_verify
    if params['openid.claimed_id'].present?
      access_card_params = @oauth.oauth_access_card_params /.*\/(\d+)$/.match(params['openid.claimed_id'])[1]

      case access_card_params[:state]
      when :ok
        sign_in access_card_params
        redirect_to profile_path
      else
        # данные не обработаны по причине access_card_params[:error]
        ActiveRecord::Base.logger.error("данные не обработаны по причине #{access_card_params[:error]}")
        # ActionCable.server.broadcast "pills", {state: 'red', message: "данные не обработаны по причине #{access_card_params[:error]}", url: authorization_url}
        redirect_to authorization_url, flash: {pills: "данные не обработаны по причине #{access_card_params[:error]}"}
      end
    else
      #code не пришёл от api соц.сети
      ActiveRecord::Base.logger.error("openid.claimed_id не пришёл от api соц.сети")
      redirect_to authorization_url, flash: {pills: 'openid.claimed_id не пришёл от api соц.сети'}

    end
  end

  private

  def set_state
    key = SecureRandom.hex(16)
    session[:oauth_state] = {key => {name: params[:name], type: params[:type]}}
  end

  # проверка пришедшего state на соответствие отправленному
  def check_state
    state_key = nil

    if params[:state].present?
      state_key = params[:state]
    elsif params['openid.state']
      state_key = params['openid.state']
    end

    unless state_key.nil?
      if session[:oauth_state] && session[:oauth_state].keys.first == state_key
        state_value = session[:oauth_state].values.first
        set_oauth state_value[:name], state_value[:type]
      else
        ActiveRecord::Base.logger.error("state #{state_key} подделан")
        redirect_to authorization_url, flash: {pills: "state #{state_key} подделан"}
      end
    else
      ActiveRecord::Base.logger.error('state не пришёл от api соц.сети')
      redirect_to authorization_url, flash: {pills: 'state не пришёл от api соц.сети'}
    end

  end

  def set_oauth name, type
    classname = "#{type}_#{name}"
    require "#{type}/#{type}"
    require "#{type}/#{classname}"

    @oauth = classname.classify.constantize.new eval("#{type}_verify_url"), session[:oauth_state]
  end

  def sign_in access_card_params
    # есть ли у нас такая скрепа
    access_card = OauthAccessCard.find_by({oauth_name: access_card_params[:oauth_name], oauth_uid: [:oauth_uid]})
    # уже авторизован и решил прикрепить скрепу!
    if @current_user_card
      # если ли она среди карточек именно этого юзера
      user_access_card = @current_user_card.oauth_access_cards.find_by({oauth_name: access_card_params[:oauth_name], oauth_uid: [:oauth_uid]})
      if user_access_card.nil?
        if access_card
          # переезд к другому юзеру
          u = access_card.user_card
          access_card.update(user_id: @current_user_card.id)
          user_access_card = access_card
          # если забрал последнюю скрепу и нет больше карточек доступа - дезинтегрируем юзера
          if u.access_cards_count == 0
            u.userable.destroy
          else
            u.set_active_card
          end
        else
          # прикрепил
          user_access_card = @current_user_card.oauth_access_cards.create access_card_params
        end
      else
        # просто заапдейтил токен и аву
        user_access_card.update(access_token: access_card_params[:access_token], token_expired: access_card_params[:token_expired], photourl: access_card_params[:photourl])
      end

      profile_update

    else
      # привет старина!
      if access_card
        user_card = access_card.user_card
        # обновляем токены, аву, авторизуем
        access_card.update(access_token: access_card_params[:access_token], token_expired: access_card_params[:token_expired], photourl: access_card_params[:photourl])
        # social_network_api :update_user_profile, sign_as, access_card # дополнение профиля пользователя данными из соц сети
        user_card.authorize! access_card
        current_user

      # привет новичок! добро пожаловать на регистрацию!
      else
        # создаём клиента
        client = Client.create
        # и его карточку доступа
        # остальные карточки доступа он заполняет в профиле
        #
        # здесь как вариант поиск юзера и сопоставление по мылу, прикрепление к нему, но это уже совсем другая история
        #
        access_card = client.user_card.oauth_access_cards.create(
            {
                oauth_name: access_card_params[:oauth_name],
                oauth_uid: access_card_params[:oauth_uid],
                access_token: access_card_params[:access_token],
                token_expired: access_card_params[:token_expired],
                photourl: access_card_params[:photourl]
            }
        )

        client.user_card.update({password: SecureRandom.hex(8)})

        client.user_card.authorize! access_card
        current_user
        profile_update

      end

    end

  end

  # дополнение профиля пользователя данными из соц сети
  def profile_update

  end

end