class OpenidController < ResponseController

  before_action :set_oauth, only: [:authorize, :verify]

  def authorize
    # отправка пользователя на подтверждение
    redirect_to @oauth.auth_request
  end

  def receiver
    if params[:code].present?
      # обмен code на токен
      access_card_params = @oauth.auth_request :step2, {:code => params[:code]}

      case access_card_params[:state]
        when :ok
          sign_in access_card_params
          redirect_to profile_path
      else
          # данные не обработаны по причине access_card_params[:error]
          ActiveRecord::Base.logger.error("данные не обработаны по причине #{access_card_params[:error]}")
          # ActionCable.server.broadcast "pills", {state: 'red', message: "данные не обработаны по причине #{access_card_params[:error]}", url: authorization_url}
          redirect_to authorization_url, flash: {pills: "данные не обработаны по причине #{access_card_params[:error]}"}
      end
    else
      #code не пришёл от api соц.сети
      ActiveRecord::Base.logger.error("code не пришёл от api соц.сети")
      # ActionCable.server.broadcast "pills", {state: 'red', message: 'code не пришёл от api соц.сети', url: authorization_url}
      redirect_to authorization_url, flash: {pills: 'code не пришёл от api соц.сети'}
    end
  end

  private

  def set_oauth
    if params[:state].present?
      classname = "openid_#{params[:state]}"
      require "openid/openid"
      require "openid/#{classname}"

      @oauth = classname.classify.constantize.new oauth_verify_url, params[:state].to_sym
    else
      ActiveRecord::Base.logger.error('state не пришёл от api соц.сети')
      # ActionCable.server.broadcast "pills", {state: 'red', message: 'state не пришёл от api соц.сети', url: authorization_url}
      redirect_to authorization_url, flash: {pills: 'state не пришёл от api соц.сети'}
    end
  end

  def sign_in access_card_params
    # есть ли у нас такая скрепа
    access_card = OauthAccessCard.find_by({oauth_name: access_card_params[:oauth_name], oauth_uid: [:oauth_uid]})
    # уже авторизован и решил прикрепить скрепу!
    if @current_user_card
      # если ли она среди карточек именно этого юзера
      user_access_card = @current_user_card.oauth_access_cards.find_by({oauth_name: access_card_params[:oauth_name], oauth_uid: [:oauth_uid]})
      if user_access_card.nil?
        if access_card
          # спизданул у другого юзера
          u = access_card.user_card
          access_card.update(user_id: @current_user_card.id)
          user_access_card = access_card
          # если забрал последнюю скрепу и нет больше карточек доступа - дезинтегрируем юзера
          if u.access_cards_count == 0
            u.userable.destroy
          else
            u.set_active_card
          end
        else
          # прикрепил
          user_access_card = @current_user_card.oauth_access_cards.create access_card_params
        end
      else
        # просто заапдейтил токен
        user_access_card.update(access_token: access_card_params[:access_token], token_expired: access_card_params[:token_expired])
      end

      profile_update

    else
      # привет старина!
      if access_card
        user_card = access_card.user_card
        # обновляем токены, авторизуем
        access_card.update(access_token: access_card_params[:access_token], token_expired: access_card_params[:token_expired])
        # social_network_api :update_user_profile, sign_as, access_card # дополнение профиля пользователя данными из соц сети
        user_card.authorize! access_card
        current_user

      # новичок! привет ссанина! добро пожаловать на регистрацию!
      else
        # создаём клиента
        client = Client.create
        # и его карточку доступа
        # остальные карточки доступа он заполняет в профиле
        #
        # здесь как вариант поиск юзера и сопоставление по мылу, прикрепление к нему, но это уже совсем другая история
        #
        access_card = client.user_card.oauth_access_cards.create(
            {
                oauth_name: access_card_params[:oauth_name],
                oauth_uid: access_card_params[:oauth_uid],
                access_token: access_card_params[:access_token],
                token_expired: access_card_params[:token_expired]
            }
        )

        client.user_card.update({password: SecureRandom.hex(8)})

        client.user_card.authorize! access_card
        current_user
        profile_update

      end

    end

  end

  # дополнение профиля пользователя данными из соц сети
  def profile_update

  end

end