class UserCard < ApplicationRecord

  belongs_to :userable, polymorphic: true

  has_secure_password

  has_many :email_access_cards, dependent: :destroy
  has_many :phone_access_cards, dependent: :destroy
  has_many :nickname_access_cards, dependent: :destroy
  has_many :oauth_access_cards, dependent: :destroy

  accepts_nested_attributes_for :nickname_access_cards, allow_destroy: true
  accepts_nested_attributes_for :email_access_cards, allow_destroy: true
  accepts_nested_attributes_for :phone_access_cards, allow_destroy: true
  accepts_nested_attributes_for :oauth_access_cards, allow_destroy: true

  after_update :online_status_broadcast

  before_validation :normalize

  validates :password, presence: true, format: {with: /.{8,}/i}, :allow_blank => true

  scope :active, -> { where(status: 1) }
  scope :passive, -> { where(status: 0) }
  scope :banned, -> { where(status: 2) }

  def active?
    self.status == 1
  end

  def passive?
    self.status == 0
  end

  def banned?
    self.status == 2
  end

  def client?
    self.access_level == 'client'
  end
# подтвердить
  def confirm
    self.update(status: 1)
  end
  # приготовить код
  def confirm_access_code
    require 'link'
    l = Link.new self.id
    pepper_email = Digest::MD5.new.update("#{CONFIG[:salt]} #{l.get} и #{self.my_email} добавим #{CONFIG[:pepper]}!").to_s
    "DIGITAL(.)(.)HEROES#{pepper_email}" #nice one рецепт от Николая Мартынова .)
  end
# забанить
  def ban!
    self.update(status: 2)
  end

  def authorize! access_card
# пользователь онлайн
    access_card.update(active: true)
# такое вот решение из-за невозможности актив рекорда за 1 раз поработать с одной моделью дважды
    CONFIG[:access_cards].each do |type, max|
      if "#{type}AccessCard".classify != access_card.class.name
        "#{type}AccessCard".classify.constantize.where(user_card_id: self.id).update(active: false)
      else
        "#{type}AccessCard".classify.constantize.where(user_card_id: self.id).where.not(id: access_card.id).update(active: false)
      end
    end

    $session_manager.authorize! self.id

    self.update({online: true})
  end

  def normalize
    self.password.strip! unless self.password.nil?
  end

  def my_active_card
    self.email_access_cards.find_by(active: true) || self.phone_access_cards.find_by(active: true) || self.nickname_access_cards.find_by(active: true) || self.oauth_access_cards.find_by(active: true) || false
  end

  def access_cards
    [{cards: self.email_access_cards, classname: 'EmailAccessCard'},
     {cards: self.nickname_access_cards, classname: 'NicknameAccessCard'},
     {cards: self.phone_access_cards, classname: 'PhoneAccessCard'},
     {cards: self.oauth_access_cards, classname: 'OAuthAccessCard'},
    ]
  end

  def access_cards_count
    self.email_access_cards.count + self.phone_access_cards.count + self.nickname_access_cards.count + self.oauth_access_cards.count
  end

  def set_active_card
    unless my_active_card
      if self.nickname_access_cards.any?
        self.nickname_access_cards.first.update(active: true)
      elsif self.email_access_cards.any?
        self.email_access_cards.first.update(active: true)
      elsif self.phone_access_cards.any?
        self.phone_access_cards.first.update(active: true)
      elsif self.oauth_access_cards.any?
        self.oauth_access_cards.first.update(active: true)
      end
    end
  end

  def online_status_broadcast
    # ActionCable.server.broadcast "radioJXL", {event: 'online_status', uid: self.userable.id, online: self.online, name: self.my_name} if saved_change_to_attribute?(:online)
  end

  def my_name
    active_card = self.my_active_card
    if (/Oauth/ =~ active_card.class.name).present?
      "#{active_card.firstname} #{active_card.lastname}"
    else
      active_card.value
    end
  end

  def self.access_cards_placeholder
    CONFIG[:access_by].map{|cardname, garbage| (I18n.t cardname, scope: 'access_cards')}.join(',')
  end

  def my_nicknames
    self.nickname_access_cards.pluck(:value)
  end

  def my_nickname
    my_nicknames.first
  end

  def my_emails
    self.email_access_cards.pluck(:value)
  end

  def my_email
    my_emails.first
  end

  def my_phones
    self.phone_access_cards.pluck(:value)
  end

  def my_phone
    my_phones.first
  end

  def whois
    self.userable.whoami
  end

  def check_access_level _alias
    CONFIG[:access_aliases][self.access_level.to_sym] >= CONFIG[:access_aliases][_alias]
  end

end