Rails.application.configure do
  # Settings specified here will take precedence over those in config/application.rb.


  # не требуется железной связи!
  Rails.application.config.active_record.belongs_to_required_by_default = false

  config.time_zone = 'Moscow'
  config.active_record.default_timezone = :local
  config.action_mailer.default_url_options = { :host => CONFIG[:domain] }

  config.i18n.load_path += Dir[Rails.root.join('config', 'locales', '*.{rb,yml}').to_s]
  config.i18n.default_locale = :ru
  config.encoding = 'utf-8'

  config.active_job.queue_adapter = :delayed_job

  config.autoload_paths += %W(#{config.root}/lib)

  config.action_view.embed_authenticity_token_in_remote_forms = true

  config.after_initialize do
    Delayed::Worker.logger = Logger.new(File.join(Rails.root, 'log', 'dj.log'))

    begin
      # запуск jobs
      if ActiveRecord::Migrator.get_all_versions.any?
        Delayed::Backend::ActiveRecord::Job.where(queue: :log_analyzer).destroy_all

        LogAnalyzerJob.set(wait_until: DateTime.now).perform_later
      end
      # уничтожение сессий
      ActiveRecord::SessionStore::Session.delete_all
    rescue Exception => error
      p error.message
    end

  end


  # In the development environment your application's code is reloaded on
  # every request. This slows down response time but is perfect for development
  # since you don't have to restart the web server when you make code changes.
  config.cache_classes = false

  # Do not eager load code on boot.
  config.eager_load = false

  # Show full error reports.
  config.consider_all_requests_local = true

  # Enable/disable caching. By default caching is disabled.
  if Rails.root.join('tmp/caching-dev.txt').exist?
    config.action_controller.perform_caching = true

    config.cache_store = :memory_store
    config.public_file_server.headers = {
      'Cache-Control' => "public, max-age=#{2.days.seconds.to_i}"
    }
  else
    config.action_controller.perform_caching = false

    config.cache_store = :null_store
  end

  # Don't care if the mailer can't send.
  config.action_mailer.raise_delivery_errors = false

  config.action_mailer.perform_caching = false

  # Print deprecation notices to the Rails logger.
  config.active_support.deprecation = :log

  # Raise an error on page load if there are pending migrations.
  config.active_record.migration_error = :page_load

  # Debug mode disables concatenation and preprocessing of assets.
  # This option may cause significant delays in view rendering with a large
  # number of complex assets.
  config.assets.debug = true

  # Suppress logger output for asset requests.
  config.assets.quiet = true

  # Raises error for missing translations
  # config.action_view.raise_on_missing_translations = true

  # Use an evented file watcher to asynchronously detect changes in source code,
  # routes, locales, etc. This feature depends on the listen gem.
  config.file_watcher = ActiveSupport::EventedFileUpdateChecker
end
