Rails.application.configure do
  # Settings specified here will take precedence over those in config/application.rb.



  # не требуется железной связи!
  Rails.application.config.active_record.belongs_to_required_by_default = false

  config.time_zone = 'Moscow'
  config.active_record.default_timezone = :local
  config.action_mailer.default_url_options = { :host => CONFIG[:domain] }

  config.i18n.load_path += Dir[Rails.root.join('config', 'locales', '*.{rb,yml}').to_s]
  config.i18n.default_locale = :ru
  config.encoding = 'utf-8'

  config.active_job.queue_adapter = :delayed_job

  config.autoload_paths += %W(#{config.root}/lib)

  config.action_view.embed_authenticity_token_in_remote_forms = true

  config.after_initialize do
    Delayed::Worker.logger = Logger.new(File.join(Rails.root, 'log', 'dj.log'))

    begin
      # запуск jobs
      if ActiveRecord::Migrator.get_all_versions.any?
        Delayed::Backend::ActiveRecord::Job.where(queue: :log_analyzer).destroy_all

        LogAnalyzerJob.set(wait_until: DateTime.now).perform_later
      end
      # уничтожение сессий
      ActiveRecord::SessionStore::Session.delete_all
    rescue Exception => error
      p error.message
    end

  end


  # Code is not reloaded between requests.
  config.cache_classes = true

  # Eager load code on boot. This eager loads most of Rails and
  # your application in memory, allowing both threaded web servers
  # and those relying on copy on write to perform better.
  # Rake tasks automatically ignore this option for performance.
  config.eager_load = true

  # Full error reports are disabled and caching is turned on.
  config.consider_all_requests_local       = false
  config.action_controller.perform_caching = true

  # Attempt to read encrypted secrets from `config/secrets.yml.enc`.
  # Requires an encryption key in `ENV["RAILS_MASTER_KEY"]` or
  # `config/secrets.yml.key`.
  config.read_encrypted_secrets = true

  # Disable serving static files from the `/public` folder by default since
  # Apache or NGINX already handles this.
  config.public_file_server.enabled = ENV['RAILS_SERVE_STATIC_FILES'].present?

  # Compress JavaScripts and CSS.
  config.assets.js_compressor = :uglifier
  # config.assets.css_compressor = :sass

  # Do not fallback to assets pipeline if a precompiled asset is missed.
  config.assets.compile = false

  # `config.assets.precompile` and `config.assets.version` have moved to config/initializers/assets.rb

  # Enable serving of images, stylesheets, and JavaScripts from an asset server.
  # config.action_controller.asset_host = 'http://assets.example.com'

  # Specifies the header that your server uses for sending files.
  # config.action_dispatch.x_sendfile_header = 'X-Sendfile' # for Apache
  # config.action_dispatch.x_sendfile_header = 'X-Accel-Redirect' # for NGINX

  config.action_cable.mount_path = '/cable/'
  # config.action_cable.url = 'wss://139.59.131.213/cable/'
  config.action_cable.allowed_request_origins = [ "https://#{CONFIG[:domain]}", /https:\/\/#{CONFIG[:domain]}.*/ ]


  # Force all access to the app over SSL, use Strict-Transport-Security, and use secure cookies.
  config.force_ssl = true

  # Force all access to the app over SSL, use Strict-Transport-Security, and use secure cookies.
  # config.force_ssl = true

  # Use the lowest log level to ensure availability of diagnostic information
  # when problems arise.
  config.log_level = :debug

  # Prepend all log lines with the following tags.
  config.log_tags = [ :request_id ]

  # Use a different cache store in production.
  # config.cache_store = :mem_cache_store

  # Use a real queuing backend for Active Job (and separate queues per environment)
  # config.active_job.queue_adapter     = :resque
  # config.active_job.queue_name_prefix = "iamrealtor_#{Rails.env}"
  config.action_mailer.perform_caching = false

  # Ignore bad email addresses and do not raise email delivery errors.
  # Set this to true and configure the email server for immediate delivery to raise delivery errors.
  # config.action_mailer.raise_delivery_errors = false

  # Enable locale fallbacks for I18n (makes lookups for any locale fall back to
  # the I18n.default_locale when a translation cannot be found).
  config.i18n.fallbacks = true

  # Send deprecation notices to registered listeners.
  config.active_support.deprecation = :notify

  # Use default logging formatter so that PID and timestamp are not suppressed.
  config.log_formatter = ::Logger::Formatter.new

  # Use a different logger for distributed setups.
  # require 'syslog/logger'
  # config.logger = ActiveSupport::TaggedLogging.new(Syslog::Logger.new 'app-name')

  if ENV["RAILS_LOG_TO_STDOUT"].present?
    ActiveRecord::Base.logger = ActiveSupport::Logger.new(STDOUT)
    ActiveRecord::Base.logger.formatter = config.log_formatter
    config.logger    = ActiveSupport::TaggedLogging.new(logger)
  end

  # Do not dump schema after migrations.
  config.active_record.dump_schema_after_migration = false

  ActionMailer::Base.smtp_settings = {
      :user_name => CONFIG[:sendgrid][:user_name],
      :password => CONFIG[:sendgrid][:password],
      :domain => CONFIG[:domain],
      :address => 'smtp.sendgrid.net',
      :port => 587,
      :authentication => :plain,
      :enable_starttls_auto => true
  }
end